#ifndef __LCONSOLE_H
#define __LCONSOLE_H

#define LINUX

#include <string.h>
#include <stdio.h>

#ifdef LINUX

    #include <termios.h>
    #include <unistd.h>
    #ifndef STDIN_FILENO
      #define STDIN_FILENO 0
    #endif

#endif

#ifdef DOS
    #include <conio2.h>
#endif


// triggers a code in Linux
#define ESC              27

// triggers a code in DOS.
#define DOSARROW        224
#define DOSZERO           0

#define ENTER            10


// These mimic
// /usr/include/X11/keysymdef.h
// but I dont want to require that .h
#define LC_KEY_UP       0xFF97
#define LC_KEY_DOWN     0xFF99
#define LC_KEY_LEFT     0xFF96
#define LC_KEY_RIGHT    0xFF98
#define LC_KEY_HOME     0xFF95
#define LC_KEY_END      0xFF9C
#define LC_KEY_INSERT   0xFF9E
#define LC_KEY_DELETE   0xFF9F
#define LC_KEY_PAGEUP   0xFF9A
#define LC_KEY_PAGEDOWN 0xFF9B


#define CODE_LEN   5


typedef struct {
    unsigned int  one_code;
    unsigned char long_codes[CODE_LEN];
    } key_code;

#ifdef LINUX
#define POS_CODES  11
const key_code KEY_CODES[POS_CODES] =
    { {ESC,             { ESC,  0,  0,   0, 0 } },
      {LC_KEY_UP,       { ESC, 91, 65,   0, 0 } },
      {LC_KEY_DOWN,     { ESC, 91, 66,   0, 0 } },
      {LC_KEY_LEFT,     { ESC, 91, 68,   0, 0 } },
      {LC_KEY_RIGHT,    { ESC, 91, 67,   0, 0 } },
      {LC_KEY_HOME,     { ESC, 91, 49, 126, 0 } },
      {LC_KEY_INSERT,   { ESC, 91, 50, 126, 0 } },
      {LC_KEY_DELETE,   { ESC, 91, 51, 126, 0 } },
      {LC_KEY_END,      { ESC, 91, 52, 126, 0 } },
      {LC_KEY_PAGEUP,   { ESC, 91, 53, 126, 0 } },
      {LC_KEY_PAGEDOWN, { ESC, 91, 54, 126, 0 } }};

#endif

#ifdef DOS
#define POS_CODES  21
const key_code KEY_CODES[POS_CODES] =
    { {ESC,             { ESC,       0, 0,   0, 0 } },
      {LC_KEY_UP,       { DOSARROW, 72, 0,   0, 0 } },  // between keys
      {LC_KEY_DOWN,     { DOSARROW, 80, 0,   0, 0 } },
      {LC_KEY_LEFT,     { DOSARROW, 75, 0,   0, 0 } },
      {LC_KEY_RIGHT,    { DOSARROW, 77, 0,   0, 0 } },
      {LC_KEY_HOME,     { DOSARROW, 71, 0,   0, 0 } },
      {LC_KEY_INSERT,   { DOSARROW, 82, 0,   0, 0 } },
      {LC_KEY_DELETE,   { DOSARROW, 83, 0,   0, 0 } },
      {LC_KEY_END,      { DOSARROW, 79, 0,   0, 0 } },
      {LC_KEY_PAGEUP,   { DOSARROW, 73, 0,   0, 0 } },
      {LC_KEY_PAGEDOWN, { DOSARROW, 81, 0,   0, 0 } },
      {LC_KEY_UP,       {  DOSZERO, 72, 0,   0, 0 } }, // The keypad keys
      {LC_KEY_DOWN,     {  DOSZERO, 80, 0,   0, 0 } },
      {LC_KEY_LEFT,     {  DOSZERO, 75, 0,   0, 0 } },
      {LC_KEY_RIGHT,    {  DOSZERO, 77, 0,   0, 0 } },
      {LC_KEY_HOME,     {  DOSZERO, 71, 0,   0, 0 } },
      {LC_KEY_INSERT,   {  DOSZERO, 82, 0,   0, 0 } },
      {LC_KEY_DELETE,   {  DOSZERO, 83, 0,   0, 0 } },
      {LC_KEY_END,      {  DOSZERO, 79, 0,   0, 0 } },
      {LC_KEY_PAGEUP,   {  DOSZERO, 73, 0,   0, 0 } },
      {LC_KEY_PAGEDOWN, {  DOSZERO, 81, 0,   0, 0 } }    };
#endif

    // Some buttons result in multiple buttons.
    // To get them all you have to do another kbhit(), readch() pair.
    // UP ARROW    = 27, 91, 65
    // DOWN ARROW  = 27, 91, 66
    // RIGHT ARROW = 27, 91, 67
    // LEFT ARROW  = 27, 91, 68
    // ESC = 27
    // INSERT = 27, 91, 50, 126
    // DELETE = 27, 91, 51, 126





const char CURSOR_COLORS_TEXT[][5] =
        {"0;30", "0;34", "0;32", "0;36", "0;31", "0;35", "0;33", "0;37",
         "1;30", "1;34", "1;32", "1;36", "1;31", "1;35", "1;33", "1;37",
           "40",   "44",   "42",   "46",   "41",   "45",   "43",   "47"};

enum CURSOR_STYLE_TYPE {
    LC_RESET     = 0,
    LC_NORMAL    = 0,
    LC_BRIGHT    = 1,
    LC_DIM       = 2,  // dont work
    LC_UNDERLINE = 3, // wont
    LC_FOUR      = 4,  // real Underline??
    LC_BLINK     = 5,   // Brights background also
    LC_SIX       = 6,  // Bright Background?
    LC_REVERSE   = 7,
    LC_HIDDEN    = 8,
    LC_NINE      = 9,
    LC_TEN       = 10
};

const int CURSOR_COLOR_LINUX_CODES[] =
    { 30, 34, 32, 36, 31, 35, 33, 37,
      30, 34, 32, 36, 31, 35, 33, 37};

enum CURSOR_COLOR_TYPE {
    LC_BLACK,
    LC_BLUE,
    LC_GREEN,
    LC_CYAN,
    LC_RED,
    LC_PURPLE,
    LC_BROWN,
    LC_LIGHT_GRAY,

    // named that have the same value, but make more sense when in Bright mode
    LC_DARK_GRAY,
    LC_LIGHT_BLUE,
    LC_LIGHT_GREEN,
    LC_LIGHT_CYAN,
    LC_LIGHT_RED,
    LC_LIGHT_PURPLE,
    LC_YELLOW,
    LC_WHITE
};




// These need to exist under both. But under DOS they will
// define to empty.
// Starts Keyboard nonblocking mode.
void SetTTYRaw(bool HideEcho);
// ends Keyboard nonblocking mode
void RestoreTTY();

// returns 1 on true 0 on false.
int IsKBHit();
// returns the button hit.
int ReadKeyboard();


void SetCursorPos(size_t X, size_t Y);
void ClearScreen();
void ResetColor();
void SetColors(CURSOR_STYLE_TYPE style, CURSOR_COLOR_TYPE Fg, CURSOR_COLOR_TYPE Bg);	  
	  
#endif // end entire header


